using System;
using System.Drawing;
using System.Diagnostics;

namespace Polygon
{
	public class Plot
	{
		PointF origin = new PointF(0.0f, 0.0f);
		Graphics g;
		double userXtoMM = 1.0;
		double userYtoMM = 1.0;
		float pageWidthDev;
		float pageHeightDev;
		float pageWidthMM;
		float pageHeightMM;
		PointF penPosition;
		Pen pen;
		SolidBrush brush;
		Font font;
		float textHeight = 3.0f;
		double axisXmin, axisXmax, axisDy;
		double axisYmin, axisYmax, axisDx;

		public Plot(Graphics g)
		{
			this.g = g;
			g.PageUnit = GraphicsUnit.Millimeter;
			pageWidthDev = g.VisibleClipBounds.Width;
			pageHeightDev = g.VisibleClipBounds.Height;
			pen = new Pen(Color.Blue, 0.2f);
			brush = new SolidBrush(Color.Blue);
			font = new Font("Arial", textHeight, GraphicsUnit.Millimeter);
		}

		public Font Font 
		{
			set { font = value; }
		}

		public void PageSizeMM(double width, double height)
		{
			float fx = 1.0f, fy = 1.0f;
			if (width > 0.0)
			{
				fx = pageWidthDev / (float) width;
				g.PageScale = fx;
			}
			if (height > 0.0)
			{
				fy = pageHeightDev / (float) height;
				g.PageScale = fy;
			}
			pageWidthMM = pageWidthDev / g.PageScale;
			pageHeightMM = pageHeightDev / g.PageScale;
			pen.Width = 0.2f / g.PageScale;

			Debug.WriteLine(String.Format("width = {0:f0}, height = {1:f2}", pageWidthMM, pageHeightMM));
			Debug.WriteLine(String.Format("pageScale = {0:f2}", g.PageScale));

		}

		public void Origin(double x, double y)
		{
			origin.X = (float) x;
			origin.Y = (float) y;
		}

		public void XScale (double xu, double xmm)
		{
			userXtoMM = xmm / xu;
		}

		public void YScale (double yu, double ymm)
		{
			userYtoMM = ymm / yu;
		}

		public PointF UserToPage(double x, double y)
		{
			float xp = (float) (x * userXtoMM) + origin.X;
			float yp = (float) (y * userYtoMM) + origin.Y;
			yp = pageHeightMM - yp;
			return new PointF(xp, yp);
		}

		public void Move(double x, double y)
		{
			penPosition = UserToPage(x, y);
		}

		public void Draw(double x, double y)
		{
			PointF newPos = UserToPage(x, y);
			g.DrawLine(pen, penPosition, newPos);
			penPosition = newPos;
		}

		public void Draw_mm(double x, double y)
		{
			PointF newPos = new PointF((float) x, pageHeightMM - (float) y);
			g.DrawLine(pen, penPosition, newPos);
			penPosition.X = (float) newPos.X;
			penPosition.Y = newPos.Y;
		}

		public void Move_mm(double x, double y)
		{
			penPosition.X = (float) x;
			penPosition.Y = pageHeightMM - (float) y;
		}

		public void DrawText_mm(string text, double x, double y)
		{
			PointF pos = new PointF((float) x, pageHeightMM - (float) y );
			g.DrawString(text, font, brush, pos);
		}

		/* 
		 * draw a x-axis
			x0, y0  starting point in mm
			xlen    length in mm 
			xmin, xmax   labels at begin and end
			dx				 label steps
			text			 axis-titel 
		*/

		public void Xaxis (double x0, double y0, double xlen, double xmin, double xmax, double dx, string text)
		{
			double scaleMultiplier = 1.0;
			string format = "{0:f0}";

			axisXmin = x0; axisXmax = x0 + xlen;  // used by grid

			double maximum = 0.5 * (xmax - xmin);
			int exponent = (int) Math.Log10(maximum) / 3;		// exponent for small and big values, engineering format
			if (exponent != 0)
				scaleMultiplier = Math.Pow(10.0 , 3.0*exponent);
			if (maximum < 0.9) exponent -= 1;
			if (maximum / scaleMultiplier < 1.0 )
				format = "{0:f1}";

			Move_mm(x0, y0);
			Draw_mm(x0 + xlen + 2.0, y0);    // draw a line for the axis

			Move_mm(x0 + xlen + 7.0, y0);
			Draw_mm(x0 + xlen + 2.0, y0 + 2.0);  // draw arrows at end of axis
			Draw_mm(x0 + xlen + 2.0, y0 - 2.0);
			Draw_mm(x0 + xlen + 7.0, y0);

			double x, xtext, y1, y2 ;
			double dxmm;	// dx in mm
			y1 = y0 - 1.0;	// y value for start and end of tic marks
			y2 = y0 + 1.0;

			dxmm = xlen * dx / (xmax - xmin);
			axisDx = dxmm;		// used by grid
			string s;			// tic-mark label
			for (x = x0, xtext = xmin; x <= x0 + xlen; x += dxmm, xtext += dx) 
			{
				Move_mm (x, y1);  
				Draw_mm (x, y2);  // draw the tic mark
				s = String.Format(format, xtext / scaleMultiplier);
				DrawText_mm(s, x - 6.0, y1 );
			}
			if (scaleMultiplier != 1.0) 
			{
				text = String.Format("{0:e*}", scaleMultiplier) + text;
			}
			DrawText_mm(text, x0 + xlen + 2.0, y0 - 1.0);

		} /* end xaxis */


		/* draw a y-axis
			x0, y0  starting point in mm
			ylen    length in mm 
			ymin, ymax   labels at begin and end
			dx				 label steps
			text			 axis-titel */

		public void Yaxis (double x0, double y0, double ylen, double ymin, double ymax, double dy, string text)
		{
			double scaleMultiplier = 1.0;
			string format = "{0:f0}";

			axisYmin = y0; axisYmax = y0 + ylen ;
			double maximum = 0.5 * (ymax - ymin);
			int exponent = (int) Math.Log10(maximum) / 3;		// exponent for small and big values, engineering format
			if (exponent != 0)
				scaleMultiplier = Math.Pow(10.0 , 3.0*exponent);
			if (maximum < 0.9) exponent -= 1;
			if (maximum / scaleMultiplier < 1.0 )
				format = "{0:f1}";

			// y-axis with arraw at the end of the axis

			Move_mm(x0,			y0);
			Draw_mm(x0,			y0 + ylen + 2.0);

			Move_mm(x0,			y0 + ylen + 7.0);
			Draw_mm(x0 - 2.0, y0 + ylen + 2.0);     // arrow
			Draw_mm(x0 + 2.0, y0 + ylen + 2.0);
			Draw_mm(x0,       y0 + ylen + 7.0);

			double y, ytext, x1, x2;
			double dymm;
			string s;
			x1 = x0 - 1.0;
			x2 = x0 + 1.0;

			dymm = ylen * dy / (ymax - ymin);  // ticmark distance in mm
			axisDy = dymm;  // used by grid
			for (y = y0, ytext = ymin; y <= y0 + ylen; y += dymm, ytext += dy) 
			{
				Move_mm (x1, y);  
				Draw_mm (x2, y);  // draw the tic mark
				s = String.Format(format , ytext / scaleMultiplier);
				DrawText_mm(s, x0 - 15.0, y + 1.5*textHeight);
			}
			// 
			if (scaleMultiplier != 1.0) 
			{
				text = String.Format("{0:e2}*", scaleMultiplier) + text;
			}
			DrawText_mm(text, x0 - 10.0, y0 + ylen + 15.0);

		} // end yaxis

	}
}
